/***************************************************************************
 *   Copyright (C) 2009 by Silicium628                                     *
 *   http://silicium628.chez-alice.fr/                                     *
 *   http://www.silicium628.fr/                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation;                                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY;                                             *
 *   See the GNU General Public License for more details.                  *
 ***************************************************************************/

/*=========================================================================================
										APE2009

par Silicium628 - voir la description de cette réalisation sur mon site
versions: voir plus bas dans la partie "const" - dernière mise à jour 26 septembre 2009
La version est visible sur l'afficheur au reset du programme
Quel plaisir de programmer un uControleur en PASCAL ! disais-je:-)
Car je précise que j'ai écrit ce firmware tout d'abord en pascal (APE2008) sous Windows :-(
Puis désirant me passer définitivement de Window$ je l'ai réécrit en C avec kontrolleurlab 
Quel plaisir donc, de programmer un uControleur en C ! et sous Linux :-))
===========================================================================================
										PRINCIPE:
--------------------------------------------------------------------------------
GENERALITES: (source Wikipedia)
--------------------------------------------------------------------------------
En une année, la Terre effectue environ 365 tours sur elle-même par rapport au Soleil,
mais effectue aussi un tour complet autour du Soleil. Donc, par rapport aux étoiles,
365 jours solaires équivalent à 366 jours sidéraux.
Les jours sidéraux sont donc un peu plus courts que les jours solaires.
La période réelle de l'année étant de 365,2422 jours solaires, la durée exacte
du jour sidéral est de : 365,2422/(365,2422+1) = 0,9972696 jour solaire, soit 23h 56m 4,09s,
soit une différence d'environ 4mn par rapport au jour solaire,
le rapport entre les deux étant de 1/360 = 0.0027 soit moins de 0.3%

En astronomie, la position d'un astre sur la sphère céleste est repérée par deux
coordonnées, l'ascension droite et la déclinaison.
A tout instant la somme de l'ascension droite d'un astre et de son angle horaire
est égale au temps sidéral.
En quelque sorte le temps sidéral mesure le déplacement de la voûte céleste en
un lieu donné par rapport au méridien local.

Unités de l'ascension droite
L'heure d'ascension droite est une unité de mesure d'angle plan et vaut 360°/24 = 15°.
La minute d'ascension droite vaut 1/60 heure d'ascension droite soit 15 minutes d'arc.
La seconde d'ascension droite vaut 1/60 minute d'ascension droite = 15 secondes d'arc.
--------------------------------------------------------------------------------
(fin de citation de Wikipedia)
--------------------------------------------------------------------------------

Une lunette astronomique immobile par rapport au sol terrestre pointe dans une
direction du ciel dont l'ascenssion droite (RA) augmente de 1s (de coordonnees
RA siderales, =15s d'arc) toutes les 1s (de temps sideral)

1 jour solaire = 24h00
1 jour sideral = 23h56'
365 * 4' = 24h (au bout d'une annee le decallage s'annule donc)
Les 4' qu'il faut ajouter au jour sideral pour obtenir un jour terrestre sont dues
au deplacement de la terre sur son orbite autour du soleil d'un jour sur l'autre

Toutes les 1s de temps, l'horloge temps reel (RTclock qui bat la seconde solaire)
incremente la variable RA_temps de 1s siderale
Si on négligeait la différence entre le temps solaire et le temps sidéral on commettrait
comme vu plus haut une erreur de 0.3% qui sur un suivi de 1h represente une erreur
de 10s de temps ou 150" d'arc (2.5' d'arc) ce qui n'est pas acceptable.
On doit donc absolument tenir compte de cette différence.
Voir la procedure "RTCtickSecond"

Deux méthodes:
--------------------------------------------------------------------------------
1) Ancienne methode: (APE2002)
Lorsque le moteur pas à pas tourne, chaque pas est compté dans la variable compte_PAS
lorsque ce nombre de pas atteint  (1/(TP_suivi_RA * 1E-6)) c'est a dire la frequence
du moteur qui est le nombre de pas par seconde en mode suivi, on decremente RA de 1;
RESULTAT: lorsque le moteur tourne a la vitesse de suivi siderale, RA ne bouge plus.
C'est le but recherche, la lunette pointant alors toujours le meme objet du ciel.
Si le moteur est mis a l'arret ou bien tourne à vitesse rapide, la variable
RA indiquera toujours l'ascension droite reellement pointee.
Note: le  "1E-6" because TP_suivi_RA est un nombre de microsecondes.
--------------------------------------------------------------------------------
2) Nouvelle methode: (depuis la version APE2008 en pascal 3.0 puis APE2009 1.0 en C)
Lorsque le moteur pas à pas tourne et est embraye, la roue codeuse incrémentale
fixée sur l'axe RA envoie des impulsions qui sont comptées par l'ext_INT_0
On calcule un angle proportionnel à ce nombre (précisement avec des valeurs
représentées par des réels en virgule flotante, merci le Pascal et ici le C) (AngleHoraire)
qui est décompté de la variable Temps_sid pour le calcul de RA (position pointée)
 ===== voir la procedure "interrupt Int0" ========
Avantage de cette seconde méthode mise en oeuvre ici: L'affichage de RA valide
même lors de déplacements rapides avec un moteur DC (=courant continu), ou manuels.

Depuis la version en pascal 4.0 un moteur DC vient seconder le moteur pas à pas pour les
déplacements rapides en ascension droite.
(la vitesse est alors de 1h de déplacement RA (15 degrés d'arc) en dix secondes, soit 360x la vitesse sidérale
Un embrayage électro-mecanique commandé automatiquement par ce soft permet de passer d'un moteur à l'autre

La vitesse maximale de déplacement en DECLI est d'environ 10 degrés d'arc en 10s,
obtenue directement par le moteur pas à pas de déclinaison.

================================================================================
Remarque concernant la liaison RS232-USB :
-Toutes les données sont transmises en mode texte et pas en binaire. En particulier
les nombres sont transmis "chiffre par chiffre" (décimal) par leur représentation en
mode texte (ASCII) et non par leur valeur directe.
Ceci pour éviter des surprises lors de transmission de valeurs correspondant à
des commandes systèmes (comme par exemple chr(10) ou chr(13)) qui sont la source de bugs.

================================================================================
L'APE2008 répond aux signaux d'une télécommande TV infra rouge universelle.
Beaucoup de modèles conviennent, il faudra adapter les codes des touches dans ce
programme le cas écheant (voir les 'case octet_IR of') dans la partie "main"
Pour ma part, j'utilise une PHILIPS type 'UNIVERSAL SBC RU 252 II'

======================== Rappel ATmega32 =======================================

Device		Flash	EEPROM	SRAM	Speed		Volts
ATmega32	32kB	1024B	2048B	0 - 16MHz	4.5 - 5.5V
================================================================================
*/



// #include <math.h>
#include <stdlib.h> 
#include <stdint.h> 
#include <stdio.h>
#include <string.h>
#include <avr/wdt.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h> 

#include <util/delay.h>

#include "dm_lcd.h"
#include "uart_628.h"


/*
  RC5Rxport      = PinD, 7, negative;
  RC5mode        = rc_6bit;   
*/


extern uint8_t asmfunction(uint8_t); // all assembly subroutines must be declared external to the C source

char * version = "3.1";
int  nbr_ast = 50; //50*19octets =950 octets; à definir en fonction de la place en EEPROM



/*
RAPPEL variables avr-gcc (vérifiable avec le .map)

char				1   -128 .. 127 ou caractères
unsigned char		1  	0 .. 255 (equiv à byte du pascal)
uint8_t				1	(c'est la même chose que l'affreux 'unsigned char') 
char toto[n]			n
int					2  	-32768 .. 32767	
int16_t				2	idem 'int'
short int			2  	pareil que int (?)
unsigned int		2	0 .. 65535
uint16_t			2	idem 'unsigned int'
long int			4   -2 147 483 648 à 2 147 483 647
int32_t				4	idem long int
long long int		8	
unsigned long int	4	0 .. 4 294 967 295  (4,2 x 10^9)
uint32_t			4	idem 'unsigned long int'
float				4
double				4	// (?)

La déclaration  char JOUR[7][9];
réserve l'espace en mémoire pour 7 mots contenant 9 caractères (dont 8 caractères significatifs).
*/


#define BUFF_SIZE 40+1

/*PARAMETRES DECLINAISON
90º -> 78,5 tours de la roue blanche -> 77.5x 7.5 =581.25 tours du moteur pas à pas
1° -> 581.25/90 = 6.458 tours du moteur pas à pas
1°=3600s -> 581.25/90*48(pas/tour)= 310 pas
310 pas -> 3600s
1 pas -> 3600/310 =11.613s
10 pas -> 116,13s
*/

#define	nb_max_PAS_DEC 10
#define	nbs_DEC 63

//==========================================================================================================
// Ces données sont définies, mémorisées et lues directement en EEPROM
  char		EEMEM LST_NOMS[50][10+1]; // liste de 50 noms (de 10 char) d'objets astronomiques (VEGA, M31 ...)
  int32_t	EEMEM LST_RA[50];
  int32_t	EEMEM LST_DECLI[50];
  int8_t	EEMEM LST_NUM_PERSO[50];
// 50x11 + 50x4 + 50x4 + 50 = 1000 octets
//==========================================================================================================
    
// données en RAM  
  
  uint8_t octet_recu;
  uint8_t no_Events;

  char buffer_rs232[BUFF_SIZE];
  char input_message[BUFF_SIZE];
  uint8_t i_buff;
  uint8_t input_message_ok;	// BOOLEAN
  uint8_t buff_overfl;	// BOOLEAN

//  delay          : SysTimer8;
  uint8_t	compteur1;
  uint8_t	compteur2;
  uint16_t	nb_ms2;
  
//  ticked         : boolean;
  int stk1;

  int32_t	memoRA_1;
  int32_t	memoRA_2;
  int32_t	memoRA_3;
  int32_t	memoRA_4;
  int32_t	memoRA_5;
  int32_t	memoRA_6;

  int32_t	RA;    		// Ascenssion droite actuellement pointée (en secondes de temps sideral)
  int32_t	RA_affi;    // valeur à afficher (non fluctuante)
  int32_t	RA_objet;   // ascension droite de l'objet en cours
  int32_t	DECLI;    	// Declinaison                      (en secondes d'arc)
  int32_t	pas_mouse;  // comptés par le codeur incrémental
  float		Angle_Horaire;     	// angle horaire mesuré par le codeur incremental (en secondes de temps sideral)
  float		Temps_sid;     		// temps sideral (relatif / selection objet) en secondes
  int32_t	G_RA;   	// goto RA                          (en secondes de temps sideral)
  int32_t	G_DECLI;   	// goto DECLI                       (en secondes d'arc)
  int32_t	sel_RA;   	// select RA                        (en secondes de temps sideral)
  int32_t	sel_DECLI;  // select DECLI                     (en secondes d'arc)
  
  uint8_t	rxAdr;
  uint8_t	rxCmd;
  uint8_t	adr1;
  uint8_t	cmd1;
  uint8_t	rxAdr_IR;
  uint8_t	rxCmd_IR; 
  uint8_t	bit_bascul_IR;	// BOOLEAN
  uint8_t	octet_IR;		//octet recu par le recepteur IR
  uint8_t	bitsix;
  uint8_t	memo_bitsix;
  uint8_t	nouveau; 		// BOOLEAN			// pour l'anti rebond signaux IR
  uint8_t	stop; 			// BOOLEAN

  uint8_t	Sens_moteur_RA;
  uint8_t	Moteur_RA ;		// Moteur_RA[1,2] definissent le numero de pas [0..3] et le bit Moteur_RA[3]=ON,OFF
  uint32_t	TP_suivi_RA;   	//Periode du moteur_RA correspondant au suivi (vitesse siderale)
  uint32_t	TPi_RA;   		//Periode du moteur_RA a atteindre
  uint32_t	TP_RA;   		//Periode du moteur_RA actuelle
  int		num_vt_RA;		//numero de la vitesse RA  [-5..+5]
  uint8_t	fx_vitesse_RA;	// [1..2..4..8..16..32]
  int16_t	cpt_PAS_RA;		// attention: signé!
  int16_t	nb_max_PAS_RA;
  uint8_t	stop_RA;		// BOOLEAN 
  uint8_t	terrestre; 		// BOOLEAN mode terrestre (arrêt du suivi - autres fonctions ok )
  uint8_t	rech_rapide; 	// BOOLEAN mode suivi astro avec vitesses de déplacement rapides
  
  uint8_t	Sens_moteurDEC;
  uint8_t	Moteur_DEC;     // Moteur_RA[1,2] definissent le numero de pas [0..3] et le bit Moteur_DEC[3]=ON,OFF
  int8_t	TP_DEC;         // Periode du moteur_DEC -remarque: OCR2 est un registre 8 bits donc TP_DEC 8 bits aussi
  int8_t	num_vt_DEC;     // signé: numero de la vitesse RA  [-5..+5]
  uint8_t	fx_vitesse_DEC;       // [1..2..4..8..16..32]
  int8_t	cpt_PAS_DEC;	// attention: signé! 
  uint8_t	stop_DEC; 		// BOOLEAN 
  uint8_t	reverse_inc_dec; 	// BOOLEAN 

  char		txt1[10+1];
  char		pos1[18+1];
  char		nom1[10+1];
  uint8_t	n_OBJ;
  uint8_t	num_odj_recu;

  uint8_t	prim_pass; 		// BOOLEAN 
  uint8_t	afficher_pos; 	// BOOLEAN 
  uint8_t	afficher_diff; 	// BOOLEAN 
  uint8_t	afficher_etat; 	// BOOLEAN     
  
  uint8_t	in232_en_cours; // BOOLEAN 
  uint8_t	sortir_astres; 	// BOOLEAN 
  uint8_t	rattraper_jeu; 	// BOOLEAN 
  uint8_t	goto_en_cours; 	// BOOLEAN 
  uint8_t	sens_ra;		// BOOLEAN
  uint8_t	sens_decli;		// BOOLEAN  
  uint8_t	goto_RA_termine; 	// BOOLEAN 
  uint8_t	goto_DECLI_termine; 	// BOOLEAN 
  uint8_t	mode_transfert; 	// BOOLEAN 
    
//  FILE		*buffer;
  
  
//--------------------------------------------------------------
//functions


  
  void init_ports(void)  // ports perso 	DDRx 0=entree, 1=sortie
{
	PORTA = 0b00000000;
	DDRA  = 0b11111111;
	
	PORTB = 0b00000000;
	DDRB  = 0b11111111;

	DDRC  = 0b11111111;
	
	DDRD  = 0b01000001;     // 7=in RC5 - 6=out LED	- PD0 = out TX RS232
}

// void init_ports(void)  // ports perso
// {
// 	PORTA = 0b00000000;
// 	DDRA = 0b11111111;
// 	PORTB = 0b00000000;
// 	DDRB = 0b11111111;
// 
// 	DDRC &= 0b01111111;     // 0=entree, 1=sortie
// 	DDRD |= 0b01000000;      // portD[6]= sortie LED
// }

  
int32_t abs_int32(int32_t x)
{
	if (x >= 0)  return x ; else return -x;
}
  
  
int32_t distance_RA(int32_t Ra1, int32_t Ra2)
// retourne la distance angulaire séparant deux RA, et donne le sens dans lequel il faut tourner pour le go_to
// voir la figure "mode de calcul RA.jpg" dans le répertoire de ce fichier source		
{
	int32_t d;
	
	d = Ra2-Ra1;
	
	if (d<0)
	{
		d = -d;
		sens_ra = 1;
	}
	else { sens_ra = 0; }
	
	if (d > 43200) 
	{
		d = 86400 -d;
		sens_ra = 1-sens_ra; // on tourne dans "l'autre" sens, qui est le chemin le plus court
	} 

	return d; 
}
  
  
uint32_t distance_DECLI(int32_t Decli1, int32_t Decli2)
// retourne la distance angulaire séparant deux DECLI, et donne le sens dans lequel il faut tourner pour le go_to
{
	uint32_t d; 
	if (Decli2 >= Decli1)  
	{
		d = (uint32_t) (Decli2-Decli1);  // une décli peut être négative ; toutefois ici d est toujours > 0
		sens_decli = 0;
	}
	else
	{
		d = (uint32_t)  (Decli1-Decli2);  // une décli peut être négative ; toutefois ici d est toujours > 0
		sens_decli = 1;
	}

	return d; 
}  
  
  
void strcpy_a_n(char *CH1, char *CH2, uint8_t a, uint8_t n )
/*
		renvoie n caractères depuis i0 de &CH2
		attention: CH2 peut ne pas être un string (pas de \0 à la fin) ce qui peut être pratique pour lire un flux
		mais a+n ne doit pas déborder de CH1 et CH2 !	
		ajoute \0 à la fin (le prévoir dans la longueur de la chaine CH1)
*/
{
	uint8_t i;
	i=0;	
	while ( i < n )
	{
		CH1[i] = CH2[a+i];
		i++;
	}
	CH1[i]='\0';
}


uint8_t isNumeric(char *CH1)
{
	uint8_t i;
	i=0;
	while (CH1[i] != '\0')
	{
		if ((CH1[i]>='0' && CH1[i]<='9') == 0) return 0;
		i++;
	}
	return 1;
}	  


void raz_buff232(void)
{
	uint8_t i;
	for (i=0; i < BUFF_SIZE; i++) { buffer_rs232[i] = '-'; }
	i_buff=0;
	buff_overfl = 0;
}   
   
  
void InitINTs(void)
/*
{Les 3 bits de poids faible de TCCR2 selectionnent l'horloge et le prescaler. voir page 119 et 125 du pdf
		Bit 6, 3  WGM21:0 = 01 -> mode CTC voir p:125 du pdfCTC = "Clear Timer on Compare Match" (CTC) Mode
		In "Clear Timer on Compare mode" (CTC mode, WGM21:0 = 2), the counter is cleared to zero when the
		counter value (TCNT2) matches the OCR2. The OCR2 defines the top value for the counter, hence also its resolution.
*/
{
	// RAZ préalable (j'ai un doute sur les valeurs par defaut...)
	GICR	= 0;
	TCCR1B	= 0;
	TCCR2	= 0;
	TIMSK	= 0;
	MCUCR	= 0;		
	
	TCCR0  |= 0b01001000;	// Timer0 en mode CTC (Clear Timer on Compare Match): WGM00=1 (bit6) et WGM01=0 (bit3) voir p:74 et 80
	TCCR0  |= 0b00000011;	// Timer0 prescaller = 1/64.	0CS0[2,1,0] = 011 voir p:82 et p:80
	TIMSK  |= 0b00000010;	// Timer/Counter0 Output Compare Match Interrupt Enable.( Bit 1 – OCIE0)  p:82
	TCCR2  |= 0b10001111;
	TIMSK  |= 0b00010000;	// Timer/Counter1, Output Compare A Match Interrupt Enable; Voir datasheet ATmega32 p:112
	TCCR1B |= 0b00000010;	// Timer1 prescaler = 1/8 il faut donc encore /2 pour avoir 1 us ; voir p:110 
	TCCR1B |= 0b00001000;	// WGM12 =1 -->  Timer1 en mode CTC 
	
	OCR0 = 250;				// Timer0 freq = 16MHz/64/250= 1kHz 
	
/* Timer2 en mode CTC p:98
	In Clear Timer on Compare or CTC mode (WGM13:0 = 4 or 12), the OCR1A or ICR1
	Register are used to manipulate the counter resolution. In CTC mode the counter is
	cleared to zero when the counter value (TCNT1) matches either the OCR1A (WGM13:0 = 4) 
	voir aussi p:109 Table 47. "Waveform Generation Mode Bit Description" mode CTC = 4
*/
	TIMSK  |= 0b10000000;  // INT Timer2 comp enable;  Voir le datasheet ATmega32.pdf p:130
	GICR   |= 0b01000000;  // extINT0 enable; voir page 67 du pdf
	MCUCR  |= 0b00000011;  // Le frond descendant de extINT0 genere une interrupt request. p:67 du pdf
}

		

void Timer1_setTime(int t)  // en microsecondes
// quartz = 16MHz et prescaller (du Timer1) = 1/8 ce qui fait 2MHz	
{		
	OCR1A = 2 * t;
}


void lcd_gotoxy_clrEOL (int x, int y)
// place le curseur en x,y et efface jusqu'a la fin de la ligne
{
	
	lcd_gotoxy(x, y);
	int i;
	for (i=x; i<(20-x); i++) 	lcd_puts(" "); 
	lcd_gotoxy(x, y);
}


uint32_t StrTo_uint32 (char *string1)
// nombres positifs  (non signés) seulement 0...4 294 967 295  (4,2 x 10^9)
// atoi m'a occasionné des problèmes... pour les grands nombres > 2 octets		
{
	uint8_t i;
	uint32_t x;
	i=0;
	x=0;

	while (string1[i] != '\0')
	{
		x=(10 * x) + (string1[i] -48);
		i++;
	}		
	return x;
}
		
		

void lcd_aff_nb (int32_t valeur1, uint8_t nb_chiffres, uint8_t position_pt_decimal, char affi_zero )
//affiche un nombre positif en representation decimale
// affi_zero = 1 affiche les zéros non significatifs
{
	int32_t N;
	uint8_t r ;
	char tbl[7];
	uint8_t i;
	char affz;

	affz = affi_zero;
	
	if ((valeur1 == 0) && (affz == 0))  // le zéro tout seul est significatif
	{
		for (i=1; i<nb_chiffres; i++) {lcd_putc(' '); }
		lcd_putc('0');
	}
	N = valeur1;
	
	for (i=1; i<=nb_chiffres; i++) 
	{
		r=48 + N % 10; // modulo (reste de la division)
		N /= 10; // quotient
		tbl[i]=r;
	}
	
	for (i=1; i<=nb_chiffres; i++)
	{
		if (i == (nb_chiffres - position_pt_decimal +1) ) { lcd_puts("."); affz=1;} // après le . les 0 seront affichés
		if (tbl[nb_chiffres+1-i] != '0') affz=1; // si  1 digit est >0 alors les 0 ultérieurs deviennent significatifs
		if ((!affz) && (tbl[nb_chiffres+1-i] == '0')) lcd_putc(' ');
		else  lcd_putc(tbl[nb_chiffres+1-i]); 
	} 
}


void lcd_aff_bin (unsigned long int valeur, int nb_digits)
//affiche un nombre en representation binaire
// 16 bits max
{
	unsigned char r ;
	char tbl[17];
	unsigned i;

	for (i=1; i<=nb_digits; i++) 
	{
		r= 48 + valeur % 2; // modulo (reste de la division)
		valeur /= 2; // quotient
		tbl[i]=r;
	};
	for (i=1; i<=nb_digits; i++)	lcd_putc(tbl[nb_digits +1 -i]);
}


void Affi_Etat(int fx_vitesse, int sens_moteur, int num_ligne)
{
	int i;
	
	char str1[3+1]; // ATTENTION:	L'indice du premier élément du tableau est 0
	
	if (afficher_etat)
	{	
		for (i=0; i<=2; i++) { str1[i] = ' '; }

		if (rech_rapide)  str1[0]= 'F';
		if (terrestre)    str1[1]= 'T';
		if (sens_moteur == 0)  str1[2]= '-';
		str1[3] = '\0';
		
		lcd_gotoxy(15,num_ligne);
		for (i=0; i<=2; i++)	lcd_putc(str1[i]);
	}	
}


void Affiche_distance_RA(uint8_t x, uint8_t y)
{
	if (afficher_diff)
	{	
		int32_t R1;
		char signe;

		R1 = distance_RA(RA, G_RA);
		if (sens_ra == 0) {signe= '-';}	else  signe= ' ';
  
		lcd_gotoxy(x,y);
//	lcd_puts("DIFF RA: ");
		lcd_putc(signe);
		lcd_aff_nb (R1,6,0,1);
	}
}


void Affiche_distance_DECLI(uint8_t x, uint8_t y)
{
	if (afficher_diff)
	{
		uint32_t R1;
		char signe;

		R1 = distance_DECLI(DECLI, G_DECLI);
		if (sens_decli == 0) {signe= '-';}	else  signe= ' ';
  
		lcd_gotoxy(x,y);
//	lcd_puts("DIFF DECLI: ");
		lcd_putc(signe);
		lcd_aff_nb (R1,6,0,1);
	}
}


void Affiche_position (char num_ligne_LCD, char nom[3+1], int32_t nb_secondes)
{
	uint8_t h_ou_d, mn, sec;
	uint32_t R1;
	char signe;

	if (afficher_pos)
	{
		if (nb_secondes < 0)
		{
			nb_secondes = -nb_secondes;
			signe = '-';
		}
		else  signe = ' ';

		h_ou_d = nb_secondes / 3600;
		R1 = nb_secondes % 3600; // % c'est la fonction modulo
		mn = R1 / 60;
		sec = R1 % 60;
		lcd_gotoxy(0, num_ligne_LCD);  
	
		lcd_puts(nom);
		lcd_putc(signe);
		lcd_aff_nb (h_ou_d,2,0,1);
		lcd_putc(':');
		lcd_aff_nb (mn,2,0,1);
		lcd_putc(':');
		lcd_aff_nb (sec,2,0,1);
	}
}



void PAS_moteur_RA(void)
/*Moteur de suivi RA
		En mode FAST, ce moteur passe la main a un moteur DC rapide , permettant des vitesses rapides...bien
		plus rapides, plusieurs centaines de fois la vitesse siderale au lieu d'un timide 32 x la vitesse siderale}
*/
//  stk1= GetStackFree;  //  test la pile (stack) en runtime
		
{
	
	if  ((Moteur_RA == 0) || (Moteur_RA > 8)) Moteur_RA = 1;

	if (!stop_RA)
	{
		if (Sens_moteur_RA == 1) 
		{
			Moteur_RA = (Moteur_RA << 1); 
			cpt_PAS_RA ++;
			if  (Moteur_RA > 8)  Moteur_RA = 1; // Moteur_RA = %0001 puis 0010 puis 0100 puis 1000 puis 0001 etc...
		}
		else
		{
			Moteur_RA = (Moteur_RA >> 1); 
			cpt_PAS_RA --;
			if  (Moteur_RA == 0) Moteur_RA = 8; //Moteur_RA =0b1000 puis 0100 puis 0010 puis 0001 puis 1000 etc...
		}
		Moteur_RA &= 0b00001111;
		PORTA &= 0b11110000; // RAZ tous les bits
		PORTA |= Moteur_RA;   // PA[0..2] utilisees pour le moteur DEC. voir schema
	}
	else  PORTA &= 0b11110000;  //stop moteur RA
	
}


void PAS_moteur_DEC(void)
/* Remarque: un même sens de rotation du moteur de déclinaison peut augmenter ou diminuer la déclinaison pointée
		suivant le quadrant où l'on se trouve
		au reset, l'APE demande dans quel quadrant on se trouve (repérés sur la monture par des points de couleur)
		voir la figure "mode de calcul DECLI.jpg" dans le répertoire de ce fichier source	
		voir aussi la fonction	"select_secteur_DECLI()" dans ce code source	
*/
		
{
	cli();
	if  ((Moteur_DEC == 0) || (Moteur_DEC > 8)) 
	{ 
		Moteur_DEC = 1;
	} 
	if (!stop_DEC) 
	{
		if (Sens_moteurDEC == 1)
		{
			Moteur_DEC = (Moteur_DEC << 1); 
			if (reverse_inc_dec == 1) {cpt_PAS_DEC++;} else {cpt_PAS_DEC--;}
			if  (Moteur_DEC > 8) { 	Moteur_DEC = 1;	} 
// Moteur_DEC = 0001 puis 0010 puis 0100 puis 1000 puis 0001 etc..
		}
		else
		{
			Moteur_DEC = (Moteur_DEC >> 1); 
			if (reverse_inc_dec == 1) {cpt_PAS_DEC--;} else {cpt_PAS_DEC++;}
			if  (Moteur_DEC == 0)  { Moteur_DEC = 8; }
 //  Moteur_DEC = 1000 puis 0100 puis 0010 puis 0001 puis 1000 etc..
		}
		
		Moteur_DEC &= 0b00001111;
		PORTA &= 0b00001111;
//    uDelay(1);
		PORTA = PORTA | (Moteur_DEC << 4); 
		// PA[4..7] utilisees pour le moteur DEC. voir schema

		if (cpt_PAS_DEC >   nb_max_PAS_DEC)  
		{ 
			DECLI = DECLI - nbs_DEC; 
			
			if (DECLI < -323990) // on vient d'atteindre la déclinaison -90 (par le bas)
			{
				stop_DEC =1;
				PORTA &=  0b00001111;  // stop moteur DEC
			}
		
			
			cpt_PAS_DEC= 0;
		} 
		if (cpt_PAS_DEC <  -nb_max_PAS_DEC)  // nb_max_PAS_DEC = 10
		{ 
			DECLI = DECLI + nbs_DEC; // nbs_DEC = 64
			
		
			if (DECLI > 323990) // on vient d'atteindre la déclinaison +90 (par le haut)
// voir la figure "mode de calcul DECLI.jpg" dans le repertoire de ce fichier source				
			{
				stop_DEC = 1;
				PORTA &=  0b00001111;  // stop moteur DEC
			}

			
			cpt_PAS_DEC= 0;
			
		}
	}
	else {	PORTA &=  0b00001111; } // stop moteur DEC
}		
		


ISR(BADISR_vect)
{
// évite de planter si une int est enable et pas de procedure associée écrite (ce qui fait reseter l'ATmega)
}


ISR (TIMER1_COMPA_vect) 
// Timer1 comp A interrupt
// voir datasheet ATmega32 p:44 pour les noms des vecteurs d'interruptions
{
	PAS_moteur_RA();
}


ISR (INT0_vect) 
/* Int0 interrupt	front descendant phase0 de la roue codeuse de souris sur RA
		LA roue codeuse incrementale
		je mesure 525 pas pour 1h00 (de graduations sidérales)
		1mn -> 525/60 =8.75 pas
		1s  ->  525/3600 = 0.1458 pas
		1 pas -> 6.857s sidérales
*/		
{
	if ((PIND & 0b00010000) > 0 ) { pas_mouse++; }
	// le niveau sur l'autre phase en quadrature determine le sens de rotation
	else { pas_mouse--; }

	Angle_Horaire = (float) pas_mouse * 6.857;  // la precision se fait ici

/* 	
	{On remarque en mode suivi que RA affiché ballade de qq decondes, c'est l'erreur
	périodique d'arrondi, sans conséquence sur le suivi, à laquelle s'ajoute un mouvement
	de plus grande periode, c'est l'erreur périodique de la mécanique du capteur
	incrémental, le moteur pas à pas tournant, lui, commme... une horloge
	Mais comme il entraine lui aussi le mouvement par un train d'engrenages,
	il y a tout de même une erreur périodique de suivi.
	On peut sérieusement envisager un suivi CCD sur étoile guide
	Vous l'aurez remarqué, tout cela n'est pas très simple, il faut jouer
	avec la précision de la mécanique et les concepts théoriques.
	Et lorsque quelque chose cloche, savoir diagnostiquer d'où cela peut bien provenir
	Les dérives se produisant sur des périodes longues, il faut savoir faire preuve
	de patience lors des mesures.
*/
}


ISR (TIMER2_COMP_vect)
// REMARQUE: The OCF2 Flag is automatically cleared when the interrupt is executed. (p:116 du pdf)
{
	PAS_moteur_DEC();
	OCR2 = TP_DEC; // OCR2 est un registre 8 bits
}




		
/*
{rappel... format message pour 1 astre
 if  signe='-' then DECLI:= -DECLI;
 le string DECLI contient exactement 7 caracteres (avec des 0 devant)
 le string RA contient exactement 5 caracteres (avec des 0 devant)
 message1:= '!E' + txt_num +'_' + 'R' + txt_RA + 'D' + signe2.Text + txt_DECLI + '_' + nom_Objet;

!E06_R21336D+0026668_BETELGEUSE
}
*/

void out_1_etoile(uint8_t num1)
		
/*
exemple:	
		!E02_R51363D0068919_ARCTURUS--
		012345678901234567890123456789			
		000000000011111111112222222222
*/
{
	uint8_t i;
	char nom_Objet[20+1]; // 20+1  	2x10
	char message1[30+1];  // 30 + 1	attention à cette longueur pour toute modif de la fonction
	unsigned int RA;
	unsigned int DECLI;
   
//	for (i=0; i<50; i++)  { LST_NOMS[i][10]='\0'; } // sinon, si des chaines sont HS, strcpy plante
  
	//strcpy(nom_Objet, LST_NOMS[num1]);


	strcat(nom_Objet,"----------");

	RA=LST_RA[num1];	//conversion volontaire de type signé en type non signé
	DECLI=LST_DECLI[num1];	//conversion volontaire de type signé en type non signé
  
	sprintf(message1,"!E%02d_R%05uldD%07uld_%-10.10s",num1, RA, DECLI, nom_Objet);

	USART_TxStr(message1);
	USART_TxByte('\n');
  
//  lcd_gotoxy(0,1);
//  lcd_puts(message1);
//  _delay_ms(500);
}


void out_astres_RS232(void)
{

	uint8_t n;

	lcd_clrscr();	
	sortir_astres = 0;
	lcd_gotoxy(0,0);
	lcd_puts("envoi Objets");  
	no_Events=1;
	for (n=1; n<=nbr_ast; n++) 
	{ 
		out_1_etoile(n);
		lcd_gotoxy(5,2);
		lcd_aff_nb (n, 2, 0, 1 );
		_delay_ms(80);
	}   
	no_Events=0;
	Affiche_position(2, "gA ", G_RA);
	Affiche_position(3, "dD ", G_DECLI);	
}

/*
void write_datas_in_EEPROM(void)
{
 	eeprom_write_block ((const void*)&LST_NOMS , (void*)0, 440);
 	eeprom_write_block ((const void*)&LST_RA , (void*)440, 160);
 	eeprom_write_block ((const void*)&LST_DECLI , (void*)600,160);
 	eeprom_write_block ((const void*)&LST_NUM_PERSO , (void*)760,40);
}


void read_datas_in_EEPROM(void)
{
 	eeprom_read_block((void*)&LST_NOMS, (const void*)0, 440); 
 	eeprom_read_block((void*)&LST_RA, (const void*)440, 160); 
 	eeprom_read_block((void*)&LST_DECLI, (const void*)600,160); 
 	eeprom_read_block((void*)&LST_NUM_PERSO, (const void*)760,40);
}
*/


void traitement_message(void)
// traitement du message RS232 recu
{	
	char		cmd1[2];
	char		numtxt[2+1];
	uint8_t		numero_objet;
	char		numPersoTxt[3+1];
	uint8_t		numPerso;
	char		str_nom[10+1];
	char 		ratxt[5+1];
	uint32_t	ra1;
	char		declitxt[7+1];
	uint32_t	decli1;
	uint8_t		i;

	in232_en_cours = 1;
	
	if (input_message_ok)
	{
		cmd1[0]=input_message[0];
		cmd1[1]=input_message[1];
		
		input_message_ok = 0;
		
		
//=====================================================================================================	
/*				
		if ((cmd1[0]=='!') && (cmd1[1]=='T'))   // Bascule en mode transfert
		{	
		mode_transfert=1;
		lcd_clrscr();		    
			//lcd_gotoxy(0,0);
		lcd_puts("== MODE TRANSFERT =="); 
	}
		else
		{
			
*/		
//=====================================================================================================	
				
		if ((cmd1[0]=='!') && (cmd1[1]=='N'))   // Bascule en mode Normal
		{	
			lcd_clrscr();
				
			afficher_pos = 1;
			afficher_diff = 1;
			mode_transfert = 0;
			Affiche_position(2, "gA ", G_RA);
			Affiche_position(3, "gD ", G_DECLI);
		}		
		
//=====================================================================================================		
		if ((cmd1[0]=='?') && (cmd1[1]=='E'))	 sortir_astres = 1; // pour envoyer le contenu des objets EEPROM
		
//=====================================================================================================			
		if ( (cmd1[0]=='!') &&(cmd1[1]=='E') ) // reception 1 objet
		{
			no_Events = 1;
			afficher_pos = 0;
			in232_en_cours = 1;
/*
exemple:	
			!E02_R51363D0068919_ARCTURUS--_106
			0123456789012345678901234567890123			
			0000000000111111111122222222223333
*/

			numtxt[0] = input_message[2];  
			numtxt[1] = input_message[3];
			numtxt[2] = '\0';	
			
			if (isNumeric(numtxt)) { numero_objet = atoi(numtxt);	}
			else {	numero_objet = 51;	}
			
			
// contrôle de la validité du message reçu
			if ((numero_objet != 0) && (numero_objet <= 50)  
						  && (input_message[5] == 'R') && (input_message[11] == 'D'))    
			{
					
				strcpy_a_n(str_nom, input_message, 20, 10);
// Ecriture du nom directement en EEPROM					
				eeprom_write_block ((const void*)&str_nom , (void*)LST_NOMS[numero_objet], 10+1);	
								
				strcpy_a_n(ratxt, input_message, 6,5);
				ratxt[5] = '\0';
				ra1=StrTo_uint32(ratxt); // le format recu n'est pas signé; ne pas utiliser atoi ici !
// Ecriture de RA directement en EEPROM	
//void eeprom_write_block (const void *pointer_ram, void *pointer_eeprom, size_t n) 
				eeprom_write_block ((void*) &ra1 , (void*) &LST_RA[numero_objet], 4); // 4 octets 	
					//eeprom_write_block ((const void*)&ra1 , (void*)LST_RA[numero_objet], 4); // 4 octets
//					eeprom_write_dword( &LST_RA[numero_objet], ra1 );
					
					
				strcpy_a_n(declitxt, input_message, 12,7); 
				declitxt[7] = '\0';
				decli1=StrTo_uint32(declitxt); // le format recu n'est pas signé; ne pas utiliser atoi ici !
					
				strcpy_a_n(numPersoTxt, input_message, 31,3); 
				numPersoTxt[3] = '\0';
				numPerso=StrTo_uint32(numPersoTxt);

// Ecriture du numéro perso de l'objet directement en EEPROM	
				eeprom_write_block ((void*) &numPerso , (void*) &LST_NUM_PERSO[numero_objet], 1); 			
					
				if (decli1 > 324000) {decli1= 324000 - decli1; } //on retrouve un nombre signé
//la déclinaison peut-être négative
// le format signé permet de faire simplement les calculs de somme et de différence 	
									
// Ecriture de la DECLI de l'objet directement en EEPROM	
				eeprom_write_block ((void*) &decli1 , (void*) &LST_DECLI[numero_objet], 4); // 4 octets		
					
				lcd_gotoxy(0,1);
				lcd_puts("RECU: ");
				lcd_puts(numtxt);
				lcd_gotoxy(0,2);
				lcd_puts(str_nom);
				lcd_gotoxy(0,3);
				lcd_puts(numPersoTxt);

			}
			else   // ABORT
			{
				afficher_pos= 1;
				lcd_clrscr();
				lcd_puts("FIN (2)");  				
				_delay_ms(300);
				lcd_clrscr();
			}

		}
					
		
	}
	raz_buff232(); 
	for (i=0; i<BUFF_SIZE; i++) {input_message[i] = '-';}
	no_Events = 0;	
	in232_en_cours = 0;
//	Affiche_position(2, "gA ", G_RA);
//    Affiche_position(3, "gD ", G_DECLI);	
//	sei();
}


void Event_1s(void)
{
		
//  ticked= 1;
	
	if (!terrestre) { Temps_sid = Temps_sid + 1.00274; }  // environ 366/365 voir le commentaire en intro
/* on ajoute 1.00274 (l'inverse de 0,9972696) et pas 0,9972696 parce que la seconde
	sidérale dure moins longtemps que la seconde solaire et donc chaque seconde solaire
	il s'écoule un peu plus d'une seconde sidérale.
	remarque: pour un suivi de la Lune, il faudra utiliser un temps Lunaire, la Lune
	étant "en retard" d'environ une heure par jour... (24h soit un tour en 21 jours) */
	
	memoRA_6 = memoRA_5;
	memoRA_5 = memoRA_4;
	memoRA_4 = memoRA_3;
	memoRA_3 = memoRA_2;
	memoRA_2 = memoRA_1;

	RA = RA_objet + (int32_t) (Temps_sid + Angle_Horaire); 
  
/*	le premier terme (RA_objet) est fixe, pour un objet selectionné
	le second terme (Temps_sid) est fonction du temps
	le troisième terme est fonction du suivi
	la somme (RA) est l'ascension droite de la position effectivement pointé
	par le télescope à un instant donné */
 

	memoRA_1 = RA;
	RA_affi = (memoRA_1 + memoRA_2 + memoRA_3 + memoRA_4 + memoRA_5 + memoRA_6) / 6; // intégration logicielle
//  ( moyenne de 6 valeurs pour éviter l'affichage fluctuant ) 
	if (RA_affi < 0)  {  RA_affi = RA_affi + 86400; } 
	if (RA_affi > 86400)  { RA_affi = RA_affi % 86400; }// modulo  
  
 // RA doit toujours être compris entre  0..24h, jamais négatif  
	if (RA < 0)  {  RA = RA + 86400; } 
	if (RA > 86400)  { RA = RA % 86400; }// modulo 
 
//  RA_affi = RA;

/*	L'erreur d'arrondi ne depasse jamais la 1/2 seconde sur le total.
	Elle est donc négligeable apres quelques centaines de secondes.
	Temps_sid est incremente par le temps et Angle_Horaire par la rotation de l'axe RA
	si le moteur est embraye, l'un compense l'autre 
*/  
	
/*	
	if ((no_Events == 0) && (mode_transfert == 1))  
	{ 
	traitement_message(); // RS232
}
	*/
	
}


void calcul_TPi_RA (void) // en fontion du numero 'num_vt_RA'
{
	int vt2;
// stk1= GetStackFree;  //  test la pile (stack) en runtime
	if (num_vt_RA == -1) { stop_RA= 1; } else stop_RA= 0;

	if (num_vt_RA >= 0) { vt2 = num_vt_RA; 	Sens_moteur_RA = 1; }
	else  {	vt2 = -num_vt_RA - 2; Sens_moteur_RA= 0; }
 	
	if (num_vt_RA != -1) {fx_vitesse_RA = 1 << vt2;} else {fx_vitesse_RA = 1;}

	if (fx_vitesse_RA != 0 )  { TPi_RA= TP_suivi_RA / fx_vitesse_RA; }  else  { TPi_RA = TP_suivi_RA; }
}


void calcul_TP_DEC (void) // en fontion du numero 'num_vt_DEC'
{
	unsigned char vt2;

	stop_DEC = (num_vt_DEC == 0) ?  1 : 0;

	if (num_vt_DEC >= 0) 	{ vt2 = num_vt_DEC; 	Sens_moteurDEC = 1;	}
	else 					{ vt2 =  - num_vt_DEC;	Sens_moteurDEC = 0;	}

	if (vt2 > 0 ) { fx_vitesse_DEC = (1 << (vt2-1)); }

	if (fx_vitesse_DEC != 0) { TP_DEC = 255 / fx_vitesse_DEC; }
	else { TP_DEC = 250; }
}


int RecvRC5 (void) 
// recepteur RC5 bas niveau qui scrute le port relié au capteur et décode le signal
/*		
		2 bits de start
		1 bit de basculement
		5 bits d'adressage (type d'appareil, TV, VCR etc...)
		6 bits de commande (touches)
*/
{
	unsigned long int t;
	unsigned char niv_entree, niv1, niv2; // booleens - niveaux des deux demi plateaux constituant un bit
	unsigned char n; 
	unsigned int data;
	
	t=0;
	niv1=0;
	niv2=0;
	data = 0;
	
	niv_entree=1;
// attente pendant la durée d'une trame d'un front descendant (le signal issu du capteur est à 1 au repos)	
	const unsigned long int tmax = 14*1778; //24892us  = durée de 1 trame (environ 25ms donc)
	
	while ( (niv_entree == 1) && (t<= tmax) ) 
	{
		if ( (PIND & 0b10000000) > 0) { niv_entree=1; } else  { niv_entree=0; }
		_delay_us(1);
		t++;
	}
/* si le signal parvient trop vite (< 2 bits) on en déduit qu'on a pris le train en marche et on saute, 
	puisque lecture impossible de la trame en cours	
*/	
	if (t < (889 * 2)) return t;
	
	if (t > tmax) return 2;	// temps maxi écoulé, pas de signal reçu
// ici on se trouve juste un epsilon après le basculement à 0 du premier bit de start (milieu de bit)		
	_delay_us(444); // durée de 1/4 bit
	
	for (n=1; n<=13; n++)
	{
		_delay_us(889); // durée de 1/2 bit
// ici on se trouve au milieu du niveau du plateau du demi bit 	
		if ( (PIND & 0b10000000) == 0) { niv1 = 0; } else { niv1 = 1; }
		_delay_us(889); // durée de 1/2 bit
// ici on se trouve au milieu du second plateau du demi bit 	
		if ( (PIND & 0b10000000) == 0) { niv2 = 0; } else { niv2 = 1; }
// ici on connait les valeurs des deux niveaux, on en déduit le sens de la transition (front montant ou descendant)
		if (niv1 == niv2) { return 3; } // erreur de lecture, bit non valide
		if ((niv1 == 1 ) && (niv2 == 0)) { data |= 0b00000001; }
		data = data << 1;
	}
	data = data >> 1; // on rejette le dernier décallage (sans perte de bit puisque largeur data = 16 bits > )
//	lcd_gotoxy(2,2);	lcd_aff_bin (data, 16);
	
	rxCmd_IR 		=  data & 0b0000000000111111;
	rxAdr_IR 		= (data & 0b0000011111000000) >> 6;
	bit_bascul_IR 	= (data & 0b0000100000000000) >> 11;
	return 0;
}


void interroge_IR(void)
{
	if (RecvRC5() == 0)     // interroge IR
	{
		octet_IR = rxCmd_IR;
		memo_bitsix= bitsix;
		bitsix= bit_bascul_IR;
		if (bitsix != memo_bitsix) nouveau=1;  else nouveau=0;
    
		PORTD |= 0b01000000;  // allume LED 
	}
	else { octet_IR = 0xFF; }
	
	PORTD &= 0b10111111;  // eteint LED 

	if (!nouveau) { octet_IR = 0xFF; }// pas de répétition auto dans cette application
}


void test_IR(void)
{
	int r1;
	
	afficher_pos= 0;
	lcd_clrscr();
	lcd_gotoxy(0, 0);
	
	lcd_puts("Test IR STOP=ok ou 0");
	
	while (rxCmd_IR !=0) 
	{	
		r1 = RecvRC5();
		if (r1==0)
		{
			lcd_gotoxy(2,1);
			lcd_puts("CMD= ");
			lcd_aff_nb (rxCmd_IR, 5, 0, 0 );
			
			lcd_gotoxy(2,2);
			lcd_puts("Adr= ");
			lcd_aff_nb (rxAdr_IR, 5, 0, 0 );
			
			lcd_gotoxy(2,3);
			lcd_puts("Bsc= ");
			lcd_aff_nb (bit_bascul_IR, 5, 0, 0 );
			
			_delay_ms(10);
		}
	}
	
	lcd_clrscr();
	afficher_pos = 1;
	
}


void select_Objet(void) // selectionne 1 etoile dans la liste en EEPROM
{
	char string1[20+1];
	char nom_objet[10+1];
	uint8_t	num_perso;

	eeprom_read_block((void*) &sel_RA, (void*) &LST_RA[n_OBJ], 4);
	eeprom_read_block((void*) &sel_DECLI, (void*) &LST_DECLI[n_OBJ], 4);			
			 
	eeprom_read_block((void*) &nom_objet, (void*) &LST_NOMS[n_OBJ], 10+1);
	eeprom_read_block((void*) &num_perso, (void*) &LST_NUM_PERSO[n_OBJ], 1);
			
	sprintf(string1,"%02d %10.10s %03d",n_OBJ, nom_objet, num_perso); //mise en forme pour l'affi
	lcd_gotoxy_clrEOL (0, 3);   
	lcd_puts(string1);

	while (octet_IR != 23)  // touche ok (à voir... suivant la télécommande)
	{
		Affiche_position(0, " A ", sel_RA);
		Affiche_position(1, " D ", sel_DECLI);

		interroge_IR();

		if (octet_IR != 255)
		{
			switch (octet_IR)   //SUIVI  réponse rapide
			{
				case 32 : 
				{
					n_OBJ++;
					if (n_OBJ > nbr_ast) {	n_OBJ = 1;	}
				}
				break;
				
				case 33 : 
				{
					n_OBJ--;
					if (n_OBJ < 1)  { n_OBJ = nbr_ast;	}
				}
				break;
			}

			eeprom_read_block((void*) &sel_RA, (void*) &LST_RA[n_OBJ], 4);
			eeprom_read_block((void*) &sel_DECLI, (void*) &LST_DECLI[n_OBJ], 4);			
			 
			eeprom_read_block((void*) &nom_objet, (void*) &LST_NOMS[n_OBJ], 10+1);
			eeprom_read_block((void*) &num_perso, (void*) &LST_NUM_PERSO[n_OBJ], 1);
			
			sprintf(string1,"%02d %10.10s %03d",n_OBJ, nom_objet, num_perso); //mise en forme pour l'affi
			lcd_gotoxy_clrEOL (0, 3);   
			lcd_puts(string1);
			
		}
	}
	
	lcd_clrscr();
	lcd_gotoxy(0, 2);
	lcd_puts("OK");
	_delay_ms(1000);
	lcd_clrscr();
	
}


void select_pos_actuelle(void)
{
	lcd_gotoxy(0, 2);
	lcd_puts("Select pos actuelle");
	_delay_ms(100);
	select_Objet();
	Temps_sid = 0;
	Angle_Horaire = 0;
	pas_mouse = 0;
	RA_objet = sel_RA;
	memoRA_6 = sel_RA;
	memoRA_5 = sel_RA;
	memoRA_4 = sel_RA;
	memoRA_3 = sel_RA;
	memoRA_2 = sel_RA;
	memoRA_1 = sel_RA;
	DECLI = sel_DECLI;
	
// réaffiche la position goto qui n'est pas réaffichée cycliquement
// contrairement à la position actuelle		
	Affiche_position(2, "gA ", G_RA); 
	Affiche_position(3, "gD ", G_DECLI);  
}


void select_pos_goto(void)
{
	lcd_gotoxy(0, 2);
	lcd_puts("Select pos GOTO");
	_delay_ms(100);
	
	select_Objet();
	
	G_RA = sel_RA;
	G_DECLI = sel_DECLI;
	
// réaffiche la position goto qui n'est pas réaffichée cycliquement
// contrairement à la position actuelle		
	Affiche_position(2, "gA ", G_RA);
	Affiche_position(3, "gD ", G_DECLI);
}


void select_secteur_DECLI(void)
{
	lcd_clrscr();
	lcd_gotoxy(0, 0);
	lcd_puts("Select secteur DECLI");
	lcd_gotoxy(0, 1);
	lcd_puts("1 - ORANGE");	
	lcd_gotoxy(0, 2);
	lcd_puts("2 - VERT");	
	
	while ((octet_IR != 1) && (octet_IR != 2))  // touche ok (à voir... suivant la télécommande)
	{		
		interroge_IR();
		if (octet_IR != 255)
		{
			switch (octet_IR)
			{
				case 1 :{reverse_inc_dec=0;}
				break;
						
				case 2 :{reverse_inc_dec=1;}
				break;		 		
			}
		}
	  	
	}
	lcd_clrscr();
}


void go_to (void)
{
	char fast_enable;
	uint32_t dst_ra;
	uint32_t dst_decli;
	uint16_t tp_max;
  
	no_Events=1;
	goto_en_cours = 1;
	fast_enable = 1;
	afficher_pos = 0; // concerne l'affichage auto par l'INT "onSysTick"
	lcd_clrscr();
	lcd_gotoxy(0, 0);  
	lcd_puts("GOTO........");
	_delay_ms(300);
	lcd_clrscr();

	lcd_gotoxy(0,0);
	lcd_puts("-*-*-*-*-");
	lcd_gotoxy(0,1);
	lcd_puts("-*-*-*-*-");
	
	Affiche_position(2, "gA ", G_RA);
	Affiche_position(3, "gD ", G_DECLI);


#define diff_Ra_min 10  
	
// touches d'annulation du goto = '0' et 'POWER'(code 12)  a voir suivant telecommande
	tp_max=0;
	goto_RA_termine = 0;
	goto_DECLI_termine = 0; 

	while ( ( (	!goto_RA_termine) || ( !goto_DECLI_termine) ) 
					&&   ((octet_IR != 0) && (octet_IR != 12) && tp_max < 12000 )  ) 
	{ 
		tp_max++; 
// ==========================================================================================			
		dst_ra = distance_RA(RA, G_RA);
		if (dst_ra < 200) { fast_enable = 0;} // ne repassera plus enable
    
    // au moins un goto (RA ou DEC en cours)
		if (dst_ra < diff_Ra_min) 
		{ //  goto RA terminé

			if (!terrestre) {num_vt_RA= 0;  } else {	num_vt_RA= -1;  }	
			calcul_TPi_RA();
			Timer1_setTime(TPi_RA);
			goto_RA_termine = 1; // disjoncte définitivement
			lcd_gotoxy(0,0);
			lcd_puts("FIN GT RA");
		}   
		if (!goto_RA_termine)    
		{ // goto RA en cours
			
			//t1 = 80 * dst_ra;
			//if (t1 < 6000)  {TPi_RA = 2000 + 6000 - t1;} else {TPi_RA = 2000;}  
			TPi_RA = 1200;  
			Timer1_setTime(TPi_RA);		
			if (sens_ra == 0)  
			{	
				Sens_moteur_RA = 0;
/*								
				num_vt_RA = -(dst / 10)-1;
				if (num_vt_RA < -5) {num_vt_RA = -5;}
				calcul_TPi_RA();
*/
				if (fast_enable) 
				{
					PORTB &= 0b11111110;
					PORTB |= 0b00000010; // sens=0 ; mise en marche moteur DC
					stop_RA = 1;
				}
				else
				{
					PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
					stop_RA = 0;
				}
			}

			if (sens_ra == 1)  
			{
				Sens_moteur_RA = 1;
	
				if (fast_enable) 
				{
					PORTB &= 0b11111110;
					PORTB |= 0b00000011; // sens=1 ; mise en marche moteur DC
					stop_RA = 1;
				}
				else
				{			
					PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
					stop_RA = 0;
				}
			}
		}
// ==========================================================================================		
		
		dst_decli = distance_DECLI(DECLI, G_DECLI);
		if (dst_decli < nbs_DEC)  
		{  // goto DEC terminé
			num_vt_DEC = 0;
			calcul_TP_DEC();
			goto_DECLI_termine = 1; // disjoncte définitivement
			lcd_gotoxy(0,1);
			lcd_puts("FIN GT DEC");
		} 
		else
		//if (goto_DECLI_termine == 0)
		{  
			stop = 0;					// goto DECLI en cours
			stop_DEC = 0;	
			if (sens_decli == 0) 
			{
				if (reverse_inc_dec == 0)	{ num_vt_DEC = 4;	/* if ( dst_decli < 200)  num_vt_DEC = 1; */}
				else { num_vt_DEC= -4;	/* if ( dst_decli < 200)  num_vt_DEC = -1; */ }
				calcul_TP_DEC();
			}
			else
			{ 
				if (reverse_inc_dec == 0)	{ num_vt_DEC= -4;	/* if ( dst_decli < 200)  num_vt_DEC = -1; */ }
				else { num_vt_DEC = 4;	/* if ( dst_decli < 200)  num_vt_DEC = 1;	*/ }
				calcul_TP_DEC();
			}
		}
// ==========================================================================================	
							
		compteur2++;
		switch (compteur2)
		{
			case	10 :  Affiche_distance_RA(12,0);
			break;
			
			case 	20 :  
			{
				Affiche_distance_DECLI(12,1);
				compteur2 = 0;
			}
			break;

		}
		_delay_ms(10);

	} 

	stop_RA = 0;
	if (!terrestre) {num_vt_RA= 0;  } else {	num_vt_RA= -1;  }
	calcul_TPi_RA(); 
	Timer1_setTime(TPi_RA);	
	num_vt_DEC = 0; 
	calcul_TP_DEC();
	afficher_pos = 1;
	PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
	_delay_ms(1000);
	goto_en_cours = 0;
	no_Events=0;
	lcd_clrscr();	
//  Affi_facteur_vitesse(fx_vitesse_RA, Sens_moteur_RA, stop_RA, 0);
}



ISR (TIMER0_COMP_vect)
// timer 1ms
{
	nb_ms2++;
	if (nb_ms2 >= 1000)
	{
		nb_ms2=0;
		Event_1s();
	}
}



ISR(USART_RXC_vect)
{
	uint8_t i;
	octet_recu = USART_RxByte();
	if (octet_recu == ';') 
	{ 
		for (i=0; i<=i_buff; i++) {	input_message[i] = buffer_rs232[i];}
		input_message_ok = 1; // sera remis à 0 que lors du traitement du message
		raz_buff232(); 
	}
	else
	{
		if (i_buff < BUFF_SIZE)
		{
			buffer_rs232[i_buff] = octet_recu;
			i_buff++;
		}
		else { buff_overfl = 1; }
	}
//	lcd_gotoxy(0,0);
//	for (i=0; i<20; i++) { lcd_putc(input_message[i]); }
}


void DECLI_plus (void)
{
	stop = 0;
	if (!terrestre && !rech_rapide) 
	{					
		num_vt_DEC++;    
		if (num_vt_DEC > 4)	 num_vt_DEC = 4;
	}
	else
	{
		if (num_vt_DEC == 0) num_vt_DEC = 4;
		else num_vt_DEC = 0; // pour ne pas passer directement de -4 à +4 ! bonjour le choc mécanique!
	}
}


void DECLI_moins (void)		
{
	stop= 0;
	if (!terrestre && rech_rapide)
	{
		num_vt_DEC--;    
		if (num_vt_DEC < - 4)  num_vt_DEC = -4;
	}
	else
	{
		if (num_vt_DEC == 0)		num_vt_DEC =  -4;
		else  num_vt_DEC = 0; // pour ne pas passer directement de +4 à -4 !
	}
					
}		
		
		
void polaire(void)
{
	RA_objet = 0;
	memoRA_6 = 0;
	memoRA_5 = 0;
	memoRA_4 = 0;
	memoRA_3 = 0;
	memoRA_2 = 0;
	memoRA_1 = 0;
	DECLI=324000;	
		
	lcd_clrscr();
	lcd_gotoxy(5,1);
	lcd_puts("POLAIRE");
	_delay_ms(1000);
	lcd_clrscr();
	Affiche_position(2, "gA ", G_RA);
	Affiche_position(3, "gD ", G_DECLI);	
	
}
		

void menu_secondaire(void)
{
	uint8_t	fin;
	
	lcd_clrscr();
	lcd_gotoxy(0,0); lcd_puts("1 - RESET");
	lcd_gotoxy(0,1); lcd_puts("2 - Mode TRANSFERT");
	lcd_gotoxy(0,2); lcd_puts("3 - POLAIRE");	
	lcd_gotoxy(0,3); lcd_puts("0 - EXIT");			
	
	fin=0;
	while ((octet_IR !=0) && !fin)
	{
		interroge_IR();		
		if (octet_IR != 255)
		{	
			switch (octet_IR)	
			{
				case 1 : 	// SOFT RESET			
				{  
					no_Events=1;
					lcd_clrscr();
					lcd_gotoxy(1,1);
					lcd_puts("SOFT RESET dans 1s");
					_delay_ms(1000); 
					wdt_enable( 10 ); 	// active le chien de garde
					while(1); 			// boucle infinie --> le chien de garde fait reseter
				} 
				break;				
				
				case 2 : 
				{				
					mode_transfert= !mode_transfert;
					if (mode_transfert)
					{
						afficher_pos = 0;
						afficher_diff = 0;
						lcd_clrscr();		    
						lcd_puts("== MODE TRANSFERT ==");
					}
					else
					{
						lcd_clrscr();
						afficher_pos = 1;
						afficher_diff = 1;
						Affiche_position(2, "gA ", G_RA);
						Affiche_position(3, "gD ", G_DECLI);		
					}
	
					fin=1;
				}
				break;				
				
				case 3 : 
				{
					polaire();
					fin=1;
				}
				break;
				
				case 0 : case 56 :
				{
					lcd_clrscr();
					Affiche_position(2, "gA ", G_RA);
					Affiche_position(3, "gD ", G_DECLI);
					fin=1;
				}
				break;
			}
		}
	}

}





int main (void)
{
	wdt_disable();
	nb_ms2=0;
	Moteur_RA = 0;
	Moteur_DEC = 0;
	
	init_ports();
	InitINTs();
	raz_buff232();
	USART_Init();
	
	compteur1 = 0;
	compteur2 = 0;
	Sens_moteur_RA = 1;
	terrestre = 0;
	rech_rapide = 0;
	goto_en_cours = 0;
	mode_transfert = 0;
	rattraper_jeu = 0;
	num_odj_recu = 0;
	

	
/*PARAMETRES RA
	Vis sans fin = roue dentée noire 34.25T -> 6h siderales
	1h sid-> 34.25/6 = 5.70833 T  (A)
	1h sid-> A*81*60/26 = 1067.02 T du Moteur (B) (60 et 26 sont le nb de dents des roues blanches du coupleur de moteurs)
	10mn sid-> B/6 = 177.836  T du Moteur (C)
	Frequence: C/600s * 1.00274 * 200 pas = 59.44127 Hz (D)  le facteur 1.00274 = 366/365 voir commentaire en intro

*/
	TP_suivi_RA= 16823;   // (soit 1/16823us = 59.44243Hz).
//le suivi peut donc se faire avec une precision ajustable à 59ppm près
	nb_max_PAS_RA= 10000000 / TP_suivi_RA; // ce qui donne round(594.424) = 594 -> donc erreur d'arrondi 0.07%

//la précision est bonne mais en contre partie l'affichage n'est mis à jour que tous les 10s
//dans une version précédente l'affichage étais mis à jour toutes les secondes, mais l'erreur était de 1.22% !

	num_vt_RA = 0;   // [-5..+5]  ( 0 = vitesse de suivi,  -1 = stop moteur)
	calcul_TPi_RA();
	Timer1_setTime(TPi_RA);
	cpt_PAS_RA = 0;
	stop_RA = 0;
	stop = 0;


	Sens_moteurDEC = 1;
	num_vt_DEC = 0;
	calcul_TP_DEC();
	stop_DEC = 1;
  
	memoRA_6 = 0;
	memoRA_5 = 0;
	memoRA_4 = 0;
	memoRA_3 = 0;
	memoRA_2 = 0;
	memoRA_1 = 0;

	RA = 0;
	RA_objet = 0;
	DECLI = 0;
	Temps_sid = 0;
	Angle_Horaire = 0;
	pas_mouse = 0;
	G_RA = 0;
	G_DECLI = 0;
	reverse_inc_dec = 0;

	lcd_init(LCD_DISP_ON);
	lcd_clrscr();
	lcd_home();
	lcd_puts("APE2009 version v");
	lcd_puts(version);
	lcd_gotoxy(7,2);
	lcd_puts("LINUX");
	_delay_ms(1000);
	
	select_secteur_DECLI();
  
	lcd_clrscr();

	afficher_pos  = 1;
	afficher_diff = 1; 
	afficher_etat = 1; 
	in232_en_cours = 0;
	sortir_astres  = 0;

	Affiche_position(2, "gA ", G_RA);
	Affiche_position(3, "gD ", G_DECLI);
	
	_delay_ms(10);	
	
	Affi_Etat(fx_vitesse_RA, Sens_moteur_RA,  2);
	Affi_Etat(fx_vitesse_DEC, Sens_moteurDEC, 3);
	
	
	Timer1_setTime(TP_suivi_RA); 
	
	// read_datas_in_EEPROM();
	
	sei(); // enable interruptions
	no_Events = 0;	
	PORTA &= 0b11111011;   // met le moteur RA en marche	
	n_OBJ = 1;

	
	compteur1=0;
	
	
	while (1)
	{
		interroge_IR();
		
		compteur1++;
		if (compteur1 > 14) {compteur1 = 0;} 

		lcd_gotoxy(13,2);						
		if ( reverse_inc_dec ) { lcd_putc('V'); } else { lcd_putc('O'); }
		
		lcd_gotoxy(13,3);
		if (mode_transfert) { lcd_putc('S'); } else { lcd_putc(' '); }

		switch (compteur1) 
		{	
			case 1 :
			{
				Affiche_position(0, " A ", RA_affi); 
				Affi_Etat(fx_vitesse_RA, Sens_moteur_RA,  2);
			}
			break;
			
			case 2 : Affiche_position(1, " D ", DECLI);
			break;
			
			case 3 : Affiche_distance_RA(12,0);
			break;	
							
			case 4 : Affiche_distance_DECLI(12,1);
			break;					
				
			case 5 : Affi_Etat(fx_vitesse_DEC, Sens_moteurDEC, 3);
			break;			
	
			case 6 : { if (mode_transfert) { traitement_message(); } } // RS232
			break;			

			case 7 : { if (sortir_astres) 	{ out_astres_RS232(); } }
			break;
		} 
		
/*	
		Le petit case of qui suit me semble plus agréable à programmer (et surtout à relire) en C 
		(ou en Pascal) plutôt qu'en assembleur! je vous dis ça pour avoir joué avec les trois!
*/	
		if (octet_IR != 255)
		{	
			switch (octet_IR)	
			{
				case 1 : select_pos_actuelle();
				break;
				
				case 2 : select_pos_goto();
				break;
				
				case 3 :    
				{
					terrestre = 1-terrestre;  // toggle mode terrestre
					if (terrestre)
					{
						num_vt_DEC= 0;
						num_vt_RA= - 1; // arret moteur RA
						calcul_TPi_RA();
						PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
					}
				}
				break;
				
				case 4 : 
				{
					lcd_clrscr();
					lcd_gotoxy(0, 1);
					lcd_puts("UN HOMME A LA MER!");
					G_RA = RA;       // fonction qui permet de revenir au même emdroit (par le GO-TO)
					G_DECLI = DECLI;
					_delay_ms(500);
					lcd_clrscr();
					Affiche_position(2, "gA ", G_RA);
					Affiche_position(3, "gD ", G_DECLI);
				}
				break;
				
				case 5 : {;} // LIBRE menu_secondaire(); 
				break;					
				
				case 6 : 
				{
					rech_rapide = 1-rech_rapide;  // toggle mode FAST
					Affi_Etat(fx_vitesse_RA, Sens_moteur_RA,  2);
					Affi_Etat(fx_vitesse_DEC, Sens_moteurDEC, 3);
				}	
				break;
				
				case 7 : 
				{
					select_secteur_DECLI(); 
					Affiche_position(2, "gA ", G_RA);
					Affiche_position(3, "gD ", G_DECLI);
				}
				break;
				
				case 8 :  // choix_saisie();
					break;
				
				case 9 : 
				{
					test_IR();
					Affiche_position(2, "gA ", G_RA);
					Affiche_position(3, "gD ", G_DECLI);
				}
				break;
				
				case 12 :
				{
					stop = 0;
					if (!terrestre)  // touche rouge (i , marche-arret)
					{
						stop_RA = 0;
						num_vt_RA = 0;
						calcul_TPi_RA();
						Timer1_setTime(TPi_RA);
						num_vt_DEC = 0;
						Affi_Etat(fx_vitesse_RA, Sens_moteur_RA, 2);
					}
					else
					{	
						num_vt_RA= - 1;			// arret moteur RA
						num_vt_DEC= 0;  		// arret moteur DECLI
						PORTB &= 0b11111100;	// stop moteur DC par ouverture du relais (PB1=0)
					}
				}
				break;
				
				case 13 : 
				{
					stop = 0;
					num_vt_RA= - 1;     	// ARRET
					PORTA |= 0b00000100;
					TP_RA = TP_suivi_RA;	// en vue de la remise en marche ulterieure
					Timer1_setTime(TP_RA);
					num_vt_DEC= 0;
					calcul_TPi_RA();
					PORTB &= 0b11111100; 	// stop moteur DC par ouverture du relais (PB1=0)
				}
				break;

				case 32  : { if (!reverse_inc_dec) { DECLI_plus();	} else { DECLI_moins();	} }
//suivant le quadrant, le moteur ne doit pas tourner dans le même sens pour s'approcher de la polaire				
				break;
				
				case 33 :  { if (!reverse_inc_dec) { DECLI_moins(); } else { DECLI_plus();  } }
				break;
				
				case 23 :   // remise en marche du SUIVI (ou STOP en mode terrestre)
				{
					stop = 0;
					num_vt_DEC=  0;    //touches ok et AV - à voir suivant télécommande
					if (!terrestre)
					{
						num_vt_RA = 0;   // moteur RA en vitesse de suivi
						PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
					}
					else
					{
						num_vt_RA = - 1; // arret moteur RA
						num_vt_DEC= 0;  // arret moteur DECLI
						PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
					}
					calcul_TPi_RA();
				}
				break;
				
				
				case (16)  : 
				{
					stop = 0;
					if ((terrestre + rech_rapide) == 0)   // Touche fleche droite
					{	
						if (num_vt_RA == 0) {num_vt_RA =  4 - 1;} 	// si vitesse de suivi
						else  {	num_vt_RA= 0; } 					// sinon vitesse de suivi
						calcul_TPi_RA();
					}	
					else // dans ce cas on discrimine entre terrestre et rech_rapide
					{
                   //--------------------------------------------------
                   // commencons par terrestre
						if ((terrestre) && (!rech_rapide)) 
						{	
							if (num_vt_RA == -1)  { 	num_vt_RA =  4 - 1; } 	// si arreté
							else  { num_vt_RA = -1; }							// sinon stop;  
							calcul_TPi_RA();	
						}
					
                   //--------------------------------------------------
                   // continuons par rech_rapide (attention: la recherche rapide fonctionne aussi en mode terrestre)
						if (rech_rapide)
						{
							if ((num_vt_RA == -1) || (num_vt_RA == 0)) 
							{
								num_vt_RA =  4 - 1;
								PORTB &= 0b11111110;
								PORTB |= 0b00000011; // sens=1 ; mise en marche moteur DC
								stop_RA = 1;
							}
							else 
							{
								num_vt_RA = 0;	//suivi;  pour ne pas passer directement de -6 à +4 !
								if (terrestre) { num_vt_RA = - 1; } //stop
								calcul_TPi_RA();
								PORTB &= 0b11111100; // stop moteur DC par ouverture du relais (PB1=0)
							}
						}
                   //--------------------------------------------------
					}
				}
				break;
				
				case (17)  : 
				{
					stop = 0;
					if ((terrestre + rech_rapide) == 0) 
					{	
						if (num_vt_RA == 0) {num_vt_RA =  -4 - 1;} 	// si vitesse de suivi
						else  {	num_vt_RA= 0; } 					// sinon vitesse de suivi
						calcul_TPi_RA();
					}	

					else // dans ce cas on discrimine entre terrestre et rech_rapide
					{	
				  		//--------------------------------------------------
                  		// commencons par terrestre
								if ((terrestre) && (!rech_rapide)) 
								{
									if (num_vt_RA == -1) {num_vt_RA =  -4 - 1;} // si arreté	} 
									else  {	num_vt_RA= -1; }
									calcul_TPi_RA();
								}
				
                   //--------------------------------------------------
                   // continuons par rech_rapide (attention: la recherche rapide fonctionne aussi en mode terrestre)
								if (rech_rapide) 
								{
									if ((num_vt_RA == -1) || (num_vt_RA == 0)) 
									{
										num_vt_RA =  -4 - 1;
										PORTB &= 0b11111110;
										PORTB |= 0b00000010; // sens=0 ; mise en marche moteur DC
										stop_RA = 1;
									}
									else 
									{
										num_vt_RA = 0;
										if (terrestre) { num_vt_RA = -1; }
										calcul_TPi_RA();
										PORTB &= 0b11111100; // stop moteur DC par ouverture du relais
									}
								}
						
					}
				}
				break;
				
				case (43) :  {;} // LIBRE // Touche rouge du bas (<<)
				break;
				
				case (47) : 
				{
					go_to();  // touche (.) tout en bas de la télécommande
					Affiche_position(2, "gA ", G_RA);
					Affiche_position(3, "gD ", G_DECLI);
				}
				break;		
							
				case 56 : menu_secondaire(); // touche (>) en bas
				break;	

				
				default : { ;}
			}
			Timer1_setTime(TPi_RA);
			calcul_TP_DEC();
		}
	}
}		
		


